# vim:set ts=2 sw=2 sts=2 et:
from rzn.api import obj
from collections import defaultdict
import re

def updatelib(libname):
  """ update all parts from library libname to the latest version """
  libs = ui.libraryManager()
  uname, lname = libname.split("/")
  librev = libs.library(lname, uname).rev()
  proj = ui.currentTab().project()
  proj.setDependencyRev(libname, librev)
  updated = 0
  for p in proj.schematic.objs(objtype=obj.SubPart):
    if p.lib == libname:
      p.reloadPart()
      if p.other():
        p.other().reloadPart()
      updated += 1
  return "updated %d parts" % updated


def updatelibs():
  """ update all parts from  all libraries to the latest versions """
  proj = ui.currentTab().project()
  for libname,_ in proj.dependencies():
    updatelib(libname)


def compactpartnames():
  """ Rename parts such that they're in order and leave no numbers unused """
  mapping = defaultdict(list)
  
  # split a "<prefix><num>" part name
  rename = re.compile("([a-zA-Z]+)(\d+)")
  def splitname(n):
    m = rename.match(n)
    if m:
      grps = m.groups()
      return grps[0], int(grps[1])
    else:
      ui.logMessage("Can't match name: %s" % n)
      return [n, 0]

  # build mapping of prefix->parts
  proj = ui.currentTab().project()
  for p in proj.schematic.objs(objtype=obj.SubPart):
    prefix, _  = splitname(p.name)
    mapping[prefix].append(p)

  # rename parts
  renamed = 0
  for prefix, parts in mapping.items():
    parts = sorted(parts, key=lambda p:splitname(p.name))
    for i, part in enumerate(parts):
      newname = "%s%d" % (prefix, i+1)
      if part.name != newname:
        ui.logMessage("Renaming %s to %s" % (part.name, newname))
        renamed += 1
        part.name = newname
  # done
  return "%d parts renamed" % renamed
