# vim:set ts=2 sw=2 sts=2 et:
from rzn.api import obj, units

def scalepolygon(scale):
  """ Scale selected polygons by scale factor """
  v = ui.currentTab().currentView()
  for o in v.selection():
    if isinstance(o, obj.Polygon):
      o.pts = tuple([int(v*scale) for v in o.pts])

def centerpolygon():
  """ Center selected polygons """
  def center(p):
    # calculate center
    npts = len(p.pts)/2
    cx = sum([p.pts[i] for i in range(0, len(p.pts), 2)]) / npts
    cy = sum([p.pts[i] for i in range(1, len(p.pts), 2)]) / npts
    # recenter
    pts = []
    for i in range(npts):
      pts.append(p.pts[i*2  ]-cx)
      pts.append(p.pts[i*2+1]-cy)
    p.pts = tuple(pts)
  # apply centering to all polygons
  v = ui.currentTab().currentView()
  for o in v.selection():
    if isinstance(o, obj.Polygon):
      center(o)

def polygon2pad(drill=units.mm(1)):
  """ Create a pad from a polygon """
  t = ui.currentTab()
  p = t.project()
  v = t.currentView()
  for o in v.selection():
    if isinstance(o, obj.Polygon):
      pad = p.layout.pad((0,0), units.mm(1), units.mm(1), "polygon", drill=drill)
      poly = []
      for x,y in o.endpoints():
        poly.extend([x,y])
      pad.polygon = tuple(poly)
      o.delete()


