# vim:set ts=2 sw=2 sts=2 et:

EPSILON = 0.000000001


def is_perp(pt1, pt2, pt3):
  dy_a = pt2[1] - pt1[1]
  dx_a = pt2[0] - pt1[0]
  dy_b = pt3[1] - pt2[1]
  dx_b = pt3[0] - pt2[0]

  if (abs(dx_a) < EPSILON) and (abs(dy_b) < EPSILON):
    return False
  if ((abs(dy_a) < EPSILON) or (abs(dy_b) < EPSILON) or
      (abs(dx_a) < EPSILON) or (abs(dx_b) < EPSILON)):
    return True
  return False


def calc_circle(pt1, pt2, pt3):
  dy_a = pt2[1] - pt1[1]
  dx_a = pt2[0] - pt1[0]
  dy_b = pt3[1] - pt2[1]
  dx_b = pt3[0] - pt2[0]

  if (abs(dx_a) < EPSILON) and (abs(dy_b) < EPSILON):
    cx = (pt2[0] + pt3[0]) / 2.
    cy = (pt1[1] + pt2[1]) / 2.
    return cx, cy

  aSlope = dy_a / float(dx_a)
  bSlope = dy_b / float(dx_b)
  if abs(aSlope-bSlope) < EPSILON: # colinear ?
    return None

  cx = (aSlope*bSlope*(pt1[1] - pt3[1]) + bSlope*(pt1[0] + pt2[0]) -
        aSlope*(pt2[0]+pt3[0])) / (2.*(bSlope-aSlope))
  cy = -1*(cx - (pt1[0]+pt2[0])/2.)/aSlope + (pt1[1]+pt2[1])/2.
  return cx, cy


def centre_from_3points(a, b, c):
  if not is_perp(a, b, c):   return calc_circle(a, b, c)
  elif not is_perp(a, c, b): return calc_circle(a, c, b)
  elif not is_perp(b, a, c): return calc_circle(b, a, c)
  elif not is_perp(b, c, a): return calc_circle(b, c, a)
  elif not is_perp(c, b, a): return calc_circle(c, b, a)
  elif not is_perp(c, a, b): return calc_circle(c, a, b)
  return None

