# vim:set ts=2 sw=2 sts=2 et:
from rzn.inputdev import mouse
from rzn.obj.objs import Pin, Pad
from .tool import Tool

class ConnTool(Tool):
  
  def __init__(self, tools):
    Tool.__init__(self, tools)
    self.ghost = None

  def _mkconn(self, ghost=False):
    view = self.view()
    pt = view.abs_pos_grid()
    t = self.objtype
    conn = view.design.make_obj(t, view.design, pt, ghost=ghost)
    if ghost:
      conn.name = "?"
    if not ghost and self.ghost and self.ghost.hasprop("angle"):
      conn.angle = self.ghost.angle
    return conn
  
  def enter(self):
    self.ghost = self._mkconn(True)
    self.view().selectObjects([self.ghost])
    if self.ghost.hasprop("angle"):
      self.set_help(self.tr("Click: place %s, Double-click: place and exit, Right-click: rotate") % self.name.lower())
    else:
      self.set_help(self.tr("Click: place %s, Double-click: place and exit") % self.name.lower())

  def exit(self):
    self.set_help("")
    if self.ghost:
      self.ghost.delete()

  def move(self, x, y, dx, dy):
    if self.ghost:
      pt = self.view().abs_pos_grid()
      self.ghost.pos = pt

  def press(self, x, y, button, mod):
    if button == mouse.LEFT:
      view = self.view()
      objs = view.design.findPoint(view.abs_pos_grid())
      if not any(isinstance(o, (Pin,Pad)) for o in objs):
        self._mkconn()
    elif button == mouse.RIGHT:
      if self.ghost and self.ghost.hasprop("angle"):
        self.ghost.angle = (self.ghost.angle+90)%360
  
  def doubleclick(self, x, y, button, mod):
    if button == mouse.LEFT:
      self.tools.switchtool("Select")

