# vim:set ts=2 sw=2 sts=2 et:
from collections import OrderedDict


class ToolException(Exception):
  pass


class Tool(object):

  def __init__(self, tools):
    self.tools = tools
    self.helptext = ""
    self.options = OrderedDict()

  def option_by_name(self, name):
    return self.options[name.replace(" ","_")]

  def add_option(self, name, typ):
    typ.name = name
    self.options[name.replace(" ","_")] = typ

  def get_option(self, name):
    tname = name.replace(" ","_")
    if tname not in self.options:
      raise ToolException("tool '%s' has no option '%s'" % (self.name, name))
    return self.options[tname].value()

  def set_option(self, name, value):
    tname = name.replace(" ","_")
    if tname not in self.options:
      raise ToolException("tool '%s' has no option '%s'" % (self.name, name))
    self.options[tname].set_value(value)

  def set_option_update(self, name, value):
    # FIXME: should be merged with set_option
    tname = name.replace(" ","_")
    if tname not in self.options:
      raise ToolException("tool '%s' has no option '%s'" % (self.name, name))
    self.options[tname].set_value(value)
    self.options[tname].choose(self.options[tname].choice())
    self.tools.notify("option_changed") 

  def _oname(self, optname):
    return "%s:%s:%s" % (self.view().name, self.name, 
                         optname.replace(" ","_"))

  def option_changed(self, optname, value):
    self.tools.notify("option_changed")
    self.view().state_set("tools", self._oname(optname), value)

  def load_option_state(self, defaults={}):
    for optname in self.options:
      v = self.view().state_get("tools", self._oname(optname))
      if v is not None:
        self.options[optname].choose(v)
      else:
        default = defaults.get(optname)
        if default is not None:
          self.options[optname].set_value(default)

  def view(self):
    return self.tools.view

  def set_help(self, msg):
    self.tools.set_help(msg)

  def tr(self, txt):
    return self.tools.translation(self.__class__.__name__, txt)

  def enter(self):
    pass

  def exit(self):
    self.text = ""
  
  def valid_actions(self):
    return ["opt_%s" % o for o in self.options]

  def cancel(self):
    self.tools.switchtool("Select")

  def press(self, x, y, button, mod):
    pass

  def release(self, x, y, button, mod):
    pass

  def doubleclick(self, x, y, button, mod):
    pass

  def move(self, x, y, dx, dy):
    pass

  def drag(self, x, y, dx, dy, button, mod):
    pass

  def selection(self):
    pass

  def change_tool(self, tool):
    self.tools.switchtool(tool)

  def valid_for(self, view, part):
    return True

  def use_prompt(self):
    return False
