# vim:set ts=2 sw=2 sts=2 et:
from rzn.inputdev import mouse
from .tool import Tool
from .toolpart import makepart
from rzn.obj.objs import Symbol, Footprint, Pin, Pad, Via
from rzn.obj.design import Schematic

class ToolDup(Tool):
  name = "Duplicate"

  def __init__(self, tools):
    Tool.__init__(self, tools)
    self.helptext = self.tr("Create copies of the selected objects")
    self.ghost = None
  
  def exit(self):
    if self.ghost:
      self.ghost.delete()
      self.ghost = None

  def move(self, x, y, dx, dy):
    if self.ghost:
      view = self.view()
      pt = view.abs_pos_grid()
      self.ghost.pos = pt
  
  def release(self, x, y, button, mod):
    if button == mouse.LEFT:
      self._place()
    elif button == mouse.RIGHT:
      if self.ghost:
        self.ghost.angle = (self.ghost.angle+90)%360

  def _place(self):
    view = self.view()
    proj = view.design.project
    schematic, layout = proj.schematic, proj.layout
    if isinstance(self.ghost, (Symbol, Footprint)):
      if isinstance(view.design, Schematic):
        f, s, name = makepart(proj, self.ghost.lib, self.ghost.part,
                              layout.findFreeSpace(), self.ghost.pos)
        s.angle = self.ghost.angle
      else:
        f, s, name  = makepart(proj, self.ghost.lib, self.ghost.part,
                               self.ghost.pos, schematic.findFreeSpace())
        f.angle = self.ghost.angle
      proj.addDesignPart(s, f, name)
    elif isinstance(self.ghost, (Pin, Pad, Via)):
      if isinstance(view.design, Schematic):
        p = schematic.make_obj(Pin, schematic, self.ghost.pos)
        p.angle = self.ghost.angle
      else:
        p = layout.make_obj(self.ghost.__class__, layout, self.ghost.pos)
        p.shape = self.ghost.shape
        p.width = self.ghost.width
        p.height = self.ghost.height
        p.drill = self.ghost.drill
        p.polygon = self.ghost.polygon
        p.name = self.ghost.name

    self.tools.switchtool("Select")

  def enter(self):
    sel = self.view().selection.all()[0]
    self.ghost = sel.clone(ghost=True)
    self.view().selectObjects([self.ghost])
    self.move(0,0,0,0)
