# vim:set ts=2 sw=2 sts=2 et:
from rzn.inputdev import mouse
from .tool import Tool
from rzn.obj import units
from rzn.obj.layers import Layers
import math

class ToolMeasure(Tool):
  name = "Measure"

  def __init__(self, tools):
    Tool.__init__(self, tools)
    self.helptext = self.tr("Measure distances")
    self.lines = []

  def enter(self):
    self.pt = None
    self.set_help(self.tr("Click to start measurement"))

  def exit(self):
    while len(self.lines):
      t = self.lines.pop()
      t.delete()
    self.set_help("")

  def press(self, x, y, button, mod):
    if button == mouse.LEFT:
      self.pt = self.view().abs_pos_grid()
  
  def _mkdistline(self, pta, ptb):
    view = self.view()
    dx, dy = pta[0]-ptb[0], pta[1]-ptb[1]
    # distance text
    dist = math.hypot(dx, dy)
    if view.render.grid.is_inches():
      unit = float(units.inch(1))
      text = "dx %0.4f\" dy %0.4f\" = %0.4f\"" % (dx/unit, dy/unit, dist/unit)
    else:
      unit = float(units.mm(1))
      text = "dx %0.3fmm dy %0.3fmm = %0.3fmm" % (dx/unit, dy/unit, dist/unit)
    self.set_help("%s" % text)
    # render line
    return [view.render.addLines(Layers.BoundBox, [pta[0], pta[1], ptb[0], ptb[1]])]

  def move(self, x, y, dx, dy):
    view = self.view()
    pt = view.abs_pos_grid()
    if self.pt:
      a,b = self.pt
      while len(self.lines):
        t = self.lines.pop()
        t.delete()
      self.lines.extend(self._mkdistline(self.pt, pt))

