# vim:set ts=2 sw=2 sts=2 et:
from rzn.inputdev import mouse
from rzn.obj.objs import (Segment, Polygon, Airwire)
from rzn.obj.objs.base import Connectable
from rzn.lib.transform import Transform, transformer
from rzn.obj.layers import Layers
from rzn.obj import funcs
from .tool import Tool

class ToolMove(Tool):
  name = "Move"
  
  def __init__(self, tools):
    Tool.__init__(self, tools)
    self.helptext = self.tr("Change the position of the selected objects")

  def enter(self, moveconns=True):
    self.set_help(self.tr("Click: place"))
    self.moveconns = moveconns
    self.targets = []
    self.ghost = None
    self._startmove()

  def exit(self):
    if self.ghost:
      self.ghost.delete()
    view = self.view()
    ep = self.endpoint if self.endpoint != -1 else None
    view.selectObjects(self.targets, ep)
    view.render.setLayerPos(Layers.Highlight, 0, 0)

  def move(self, x, y, dx, dy):
    view = self.view()
    pt = view.abs_pos_grid()
    if (self.endpoint != -1) and (len(self.targets) == 1):
      funcs.set_endpoint(self.ghost, self.endpoint, pt)
    else:
      dx, dy = (pt[0]-self.startpt[0]), (pt[1]-self.startpt[1])
      view.render.setLayerPos(Layers.Highlight, dx, dy)

  def _buildpinmap(self):
    extern = set()
    sel = set(self.targets)

    def addconns(obj):
      for curpoint in obj.points():
        for o in obj.base.findPoint(curpoint):
          if o == obj or not isinstance(o, Connectable):
            continue
          for otherpoint in o.points():
            if otherpoint == curpoint:
              if o not in sel:
                extern.add((o, curpoint))
              break

    for o in self.targets:
      addconns(o)
    return extern

  def _finishmove(self):
    view = self.view()
    pt = view.abs_pos_grid()
    dx, dy = (pt[0]-self.startpt[0]), (pt[1]-self.startpt[1])
    # for each connectable point on the target, move all other points
    # connected to it
    for o in self.targets:
      funcs.move_rel(o, pt, dx, dy, self.endpoint, self.moveconns, self.targets)
    if self.rotation != 0:
      pinmap = self._buildpinmap()
      if (len(self.targets) == 1) and self.targets[0].hasprop("angle"):
        cx,cy = self.targets[0].pos
      else:
        cx,cy = transformer.centroid(self.targets)
      tfm = Transform.ident().trans(cx,cy).rot(self.rotation).trans(-cx,-cy)
      transformer.rotate(self.targets, self.rotation)
      for obj, point in pinmap:
        if isinstance(obj, Segment):
          if obj.pta == point:
            obj.pta = tuple(tfm.points(obj.pta))
          elif obj.ptb == point:
            obj.ptb = tuple(tfm.points(obj.ptb))

  def _startmove(self):
    allobj = self.view().selection.all()
    selpoint = self.view().selection.point
    self.targets = [o for o in allobj if not isinstance(o, Airwire)]
    self.view().selectObjects(self.targets, selpoint)
    self.centroid = transformer.centroid(allobj)
    self.rotation = 0
    if selpoint is not None:
      assert isinstance(self.targets[0], (Segment, Polygon))
      self.startpt = self.view().abs_pos()
      self.endpoint = selpoint
      self.ghost = self.targets[0].clone(ghost=True)
      self.view().selectObjects([self.ghost], selpoint)
    else:
      self.startpt = self.view().abs_pos_grid()
      self.endpoint = -1

  def _rotate(self):
    view = self.view()
    layer = view.render.layers[Layers.Highlight]
    if (len(self.targets) == 1) and self.targets[0].hasprop("angle"):
      cx,cy = self.targets[0].pos
    else:
      cx,cy = self.centroid
    layer.applyTransform(lambda t: t.trans(cx,cy).rot(90).trans(-cx,-cy))
    self.rotation = (self.rotation + 90) % 360
    view.render.dirty = True

  def press(self, x, y, button, mod):
    if button == mouse.LEFT:
      self._finishmove()

  def release(self, x, y, button, mod):
    if button == mouse.LEFT:
      self.tools.switchtool("Select")
    elif button == mouse.RIGHT:
      if self.endpoint == -1:
        self._rotate()

