# vim:set ts=2 sw=2 sts=2 et:
from .conntool import ConnTool
from rzn.obj.objs import Pad
from . import options

class ToolPad(ConnTool):
  name = "Pad"
  objtype = Pad
  
  def __init__(self, tools):
    ConnTool.__init__(self, tools)
    self.helptext = self.tr("Create a pad")
    self.add_option("Layer", options.Layer(self, "CopperTop", layershow="copper",
                             helptext=self.tr("Set pad layer")))
    self.add_option("Width", options.Dimension(self, defaultidx=3,
                             helptext=self.tr("Set pad width")))
    self.add_option("Height", options.Dimension(self, defaultidx=3,
                              helptext=self.tr("Set pad height")))
    self.add_option("Diameter", options.Dimension(self, defaultidx=2,
                                helptext=self.tr("Set pad drill diameter")))
    self.add_option("Shape", 
        options.Choice(self, ["circle", "rectangle", "octagon"],
                       helptext=self.tr("Set pad shape")))
    self.add_option("Stencil Scale", options.Scale(self, 
      helptext=self.tr("Scale factor for the stencil mask")))
    self.add_option("Stop Scale", options.Scale(self, 
      helptext=self.tr("Scale factor for the stop mask")))

  def _mkconn(self, ghost=False):
    o = ConnTool._mkconn(self, ghost)
    o.shape = self.get_option("Shape")
    o.layer = self.get_option("Layer")
    o.width = self.get_option("Width")
    o.height = self.get_option("Height")
    o.drill = self.get_option("Diameter")
    o.stencil = (o.drill == 0)
    o.stopscale = self.get_option("Stop Scale")
    o.stencilscale = self.get_option("Stencil Scale")
    return o
  
  def option_changed(self, optname, value):
    ConnTool.option_changed(self, optname, value)
    if self.ghost:
      self.ghost.delete()
      self.ghost = self._mkconn(True)
  
  def valid_for(self, view, part):
    return part and (view == "layout")
