# vim:set ts=2 sw=2 sts=2 et:
from rzn.inputdev import mouse
from .tool import Tool
from rzn.obj.objs import Polygon
from . import options

class ToolPolygon(Tool):
  name = "Polygon"
  
  def __init__(self, tools):
    Tool.__init__(self, tools)
    self.helptext = self.tr("Draw polygons")
    self.add_option("Layer", options.Layer(self, "CopperTop",
                                           helptext=self.tr("Set polygon layer")))
    self.ghost = None

  def option_changed(self, optname, value):
    Tool.option_changed(self, optname, value)
    if optname == "Layer":
      if self.ghost:
        self.ghost.delete()
        self.ghost = self._mkpoly(self.pts, ghost=True)

  def enter(self):
    self.pts = []
    self.ghost = None
    self.view().render.setLayerVisible(self.get_option("Layer"), True)
    self.set_help(self.tr("Click: start polygon"))

  def exit(self):
    self.set_help("")
    if self.ghost:
      self.ghost.delete()
    if len(self.pts) > 2:
      self._mkpoly(self.pts)
  
  def _mkpoly(self, points, ghost=False):
    view = self.view()
    pts = []
    x0, y0 = points[0]
    for x,y in points:
      pts.append(x-x0)
      pts.append(y-y0)
    p = view.design.make_obj(Polygon, view.design, (x0, y0), tuple(pts), ghost=ghost)
    p.layer = self.get_option("Layer")
    return p
  
  def move(self, x, y, dx, dy):
    if self.ghost:
      view = self.view()
      pt = view.abs_pos_grid()
      if len(self.pts) and (pt != self.pts[-1]):
        self.ghost.delete()
        self.ghost = self._mkpoly(self.pts+[pt], ghost=True)

  def release(self, x, y, button, mod):
    if button == mouse.LEFT:
      pt = self.view().abs_pos_grid()
      # add point
      if len(self.pts) == 0:
        self.pts.append(pt)
      elif (len(self.pts) > 2) and pt == self.pts[0]:
        if self.ghost:
          self.ghost.delete()
          self.ghost = None
        self.tools.switchtool("Select")
        return
      elif len(self.pts) > 1 or self.pts[-1] != pt:
        # handle a startpoint on the middle of a line
        self.pts.append(pt)

      if len(self.pts) >= 1:
        if self.ghost:
          self.ghost.delete()
        self.ghost = self._mkpoly(self.pts, ghost=True)
      self.set_help(self.tr("Click: create points"))

  def valid_for(self, view, part):
    return view == "layout"
