# vim:set ts=2 sw=2 sts=2 et:

from .tool import Tool
from rzn.obj.objs import Segment, Airwire
from rzn.obj.objs.common import nearestPointOnLine
from rzn.obj.layers import Layers

class ToolRtree(Tool):
  name = "Rtree"

  def __init__(self, tools):
    Tool.__init__(self, tools)
    self.lines = []

  def exit(self):
    while len(self.lines):
      t = self.lines.pop()
      t.delete()

  def move(self, x, y, dx, dy):
    view = self.view()
    pt = view.abs_pos()
    knn = view.design.rtree.search_knn(*pt, k=3)
    a,b = pt
    while len(self.lines):
      t = self.lines.pop()
      t.delete()
    for x in knn:
      if isinstance(x.obj, (Segment, Airwire)):
        c,d = nearestPointOnLine((a,b), x.obj.pta, x.obj.ptb)
      else:
        c,d = x.obj.pos
      t = view.render.addLines(Layers.BoundBox, [a,b,c,d])
      self.lines.append(t)
      t = view.render.addLines(Layers.BoundBox, [x.x1,x.y1,x.x2,x.y1,
                                   x.x2,x.y1,x.x2,x.y2,
                                   x.x2,x.y2,x.x1,x.y2,
                                   x.x1,x.y2,x.x1,x.y1])
      self.lines.append(t)

