# vim:set ts=2 sw=2 sts=2 et:
from .tool import Tool
from rzn.inputdev import mouse
from rzn.obj.objs import Text
from rzn.obj.layers import Layers
from . import options

class ToolText(Tool):
  name = "Text"

  def __init__(self, tools):
    Tool.__init__(self, tools)
    self.helptext = self.tr("Add text strings to the design")
    sch = (self.view().name == "schematic")
    self.add_option("Layer", options.Layer(self, "Label" if sch else "NameTop",
                             helptext=self.tr("Set text layer")))
    self.add_option("Size", options.Dimension(self, defaultidx=2,
                            helptext=self.tr("Set text size")))
    self.add_option("Width", options.Dimension(self, defaultidx=0,
                             helptext=self.tr("Set text weight (line width)")))
    self.ghost = None
  
  def option_changed(self, optname, value):
    Tool.option_changed(self, optname, value)
    if self.ghost:
      self.ghost.delete()
      self.ghost = self._mktxt(self.view().abs_pos_grid(), ghost=True)

  def enter(self):
    self.text = self.tools.show_text_entry()
    # check if layer is visible
    if not self.view().render.getLayerVisible(self.get_option("Layer")):
      # if it's a layout layer, change it to the visible one
      if self.get_option("Layer") == Layers.NameTop:
        self.set_option("Layer", Layers.NameBottom)
      elif self.get_option("Layer") == Layers.NameBottom:
        self.set_option("Layer", Layers.NameTop)

    if self.text is None:
      self.tools.switchtool("Select")
    else:
      self._maketext()
  
  def exit(self):
    if self.ghost:
      self.ghost.delete()
      self.ghost = None
    Tool.exit(self)

  def move(self, x, y, dx, dy):
    if self.ghost:
      self.ghost.pos = self.view().abs_pos_grid()

  def press(self, x, y, button, mod):
    if button == mouse.LEFT:
      self._place()
    elif button == mouse.RIGHT:
      if self.ghost:
        self.ghost.angle = (self.ghost.angle+90)%360

  def _mktxt(self, pos, ghost=False):
    view = self.view()
    t = view.design.make_obj(Text, view.design, pos, self.text, ghost=ghost)
    t.layer = self.get_option("Layer")
    t.mirror = ("Bottom" in Layers.names[t.layer])
    t.size = self.get_option("Size")/500.0
    t.width = self.get_option("Width")
    if self.ghost:
      t.angle = self.ghost.angle
    return t

  def _place(self):
    if len(self.text) and self.ghost:
      self._mktxt(self.ghost.pos)

  def _maketext(self):
    pt = self.view().abs_pos_grid()
    self.ghost = self._mktxt(pt, ghost=True)
    self.view().selectObjects([self.ghost])
    self.move(0,0,0,0)
    return False

