# vim:set ts=2 sw=2 sts=2 et:
from .conntool import ConnTool
from rzn.obj.objs import Via, Polygon
from . import options

class ToolVia(ConnTool):
  name = "Via"
  helptext = "Create a via"
  objtype = Via
  
  def __init__(self, tools):
    ConnTool.__init__(self, tools)
    self.helptext = self.tr("Create a via")
    self.add_option("Via Diameter", options.Dimension(self, defaultidx=3,
                                helptext=self.tr("Set via diameter")))
    self.add_option("Via Drill", options.Dimension(self, defaultidx=2,
                                helptext=self.tr("Set drill diameter")))
    self.add_option("Via Mask", 
        options.Choice(self, ["Tented", "Drill", "Open"],
                       helptext=self.tr("Set via masking style")))

  def _getname(self):
    pt = self.view().abs_pos_grid()
    objs = self.view().design.findPoint(pt)
    for o in objs:
      if isinstance(o, Polygon):
        if o.point_in_clipped(pt):
          return o.net
    return None

  def _mkconn(self, ghost=False):
    o = ConnTool._mkconn(self, ghost)
    o.width = self.get_option("Via Diameter")
    o.height = self.get_option("Via Diameter")
    o.drill = self.get_option("Via Drill")
    vstyle = self.get_option("Via Mask")
    if vstyle == "Tented":
      o.stoptop = o.stopbot = o.drill
      o.stop = False
    elif vstyle == "Drill":
      o.stoptop = o.stopbot = o.drill
    elif vstyle == "Open":
      o.stoptop = o.stopbot = o.width
    name = self._getname()
    if name:
      o.name = name
    return o
  
  def option_changed(self, optname, value):
    ConnTool.option_changed(self, optname, value)
    if self.ghost:
      self.ghost.delete()
      self.ghost = self._mkconn(True)
  
  def valid_for(self, view, part):
    return (not part) and (view == "layout")
