# vim:set ts=2 sw=2 sts=2 et:
from rzn.inputdev import mouse
from .toolline import ToolLine
from rzn.obj.objs.common import distPointToLine, distPointToPoint
from rzn.obj.objs import Net, Symbol
from rzn.obj.layers import Layers
from rzn.obj import units

class ToolWire(ToolLine):
  name = "Wire"
  objcls = Net
  
  def __init__(self, tools):
    ToolLine.__init__(self, tools)
    self.helptext = self.tr("Draw nets")

  def enter(self):
    ToolLine.enter(self)
    self.netname = None
    self.set_option("Layer", Layers.Net)
    self.set_help(self.tr("Click: start wire"))
    self.allpts = []
  
  def _update_help(self):
    msg =  (self.tr(" on net '%s'") % self.netname) if self.netname else ""
    self.set_help(self.tr("Click: create wire segment%s, Right-click: change draw style") % msg)

  def _get_point_and_net(self):
    view = self.view()
    gpt = pt = view.abs_pos_grid()
    # check if we're ending on an existing net
    for o in view.design.findPoint(pt):
      if isinstance(o, Net) and distPointToLine(pt, o.pta, o.ptb) == 0:
        self.netname = o.name
        return pt, o, True
    # check if we're clicking on a symbol pin not on the grid
    pt = view.abs_pos()
    for o in view.design.findPoint(pt):
      if isinstance(o, Symbol):
        for pos, conn in o.connectors():
          if distPointToPoint(pos, pt) < units.mm(1):
            # if we're touching a forcenet pin, use it as the net name
            if conn.forcenet and not self.netname:
              self.netname = conn.name
            return pos, None, True
    # otherwise just use standard grid point
    return gpt, None, False

  def _splitseg(self, seg, pt):
    if not ((pt == seg.pta) or (pt == seg.ptb)):
      s1 = self._mkseg(seg.pta, pt)
      s2 = self._mkseg(pt, seg.ptb)
      s1.name = s2.name = self.netname
      s1.width = s2.width = seg.width
      seg.delete()

  def press(self, x, y, button, mod):
    if button == mouse.LEFT:
      pt, endOnNet, canStop = self._get_point_and_net()
      # add point
      if len(self.pts) == 0 or self.pts[-1] != pt:
        # handle a startpoint on the middle of a line
        if endOnNet:
          self._splitseg(endOnNet, pt)
        self.pts.append(pt)
        self.allpts.append(pt)
        self._delete_ghosts()
      # if we have 2, make a segment
      if len(self.pts) == 2:
        pa, pb = self.pts
        self.pts.pop(0)
        s = self._mksegs(pa, pb, name=self.netname)
        if self.netname is None:
          self.netname = s[0].name
        # rename all nets 
        # FIXME: doesn't work renaming all nets with the same name not created
        # in this tool invocation
        if endOnNet:
          for seg in self.segs:
            seg.name = self.netname
        self.segs.extend(s)
      self._update_help()
      if canStop and len(self.allpts) > 1:
        self.tools.switchtool("Select")
    elif button == mouse.RIGHT:
      self.options["Style"].cycle()

  def valid_for(self, view, part):
    return (not part) and (view == "schematic")
