# vim:set ts=2 sw=2 sts=2 et:
from rzn.api import VCPlugin, VCException, Logger
import os
import re
from . import raw, hgrc
from .hgfile import HgPluginFile

log = Logger(__name__)

class HgVCPlugin(VCPlugin):
  def __init__(self):
    VCPlugin.__init__(self, "Hg", HgPluginFile)

  def usable(self):
    try:
      out, ret = raw.hg("version")
      m = re.match(".*Mercurial.*\([^ ]+ (\d(\.\d+){1,3}).*\)", out)
      if m:
        v = m.groups()[0]
        log.info("HG v%s" % v)
        return True
      log.error("No version info in HG response: %s" % out)
    except Exception as e: 
      log.error("couldn't find hg: %s" % str(e))
    return False

  def create(self, basedir):
    HGIGNORE = "syntax: glob\nlocal\n"
    os.makedirs(basedir)
    log.info("creating repo %s" % basedir)
    raw.hg("init", basedir)
    self.vcfile(basedir, "README.md").saveraw("", False, "")
    self.vcfile(basedir, ".hgignore").saveraw(HGIGNORE, False, "")
    self.vcfile(basedir, ".razenrepo").saveraw("hg", False, "")
    self.setUsername(basedir)
    return basedir

  def clone(self, src, destdir):
    out, ret = raw.hg("clone", src, destdir)
    if ret:
      log.error("failed to clone %s: %s" % (src, out))
      raise VCException("Failed to clone %s to %s" % (src, destdir))
    self.setUsername(destdir)
  
  def setUsername(self, basedir):
    user = self.getConfig("user")
    email = self.getConfig("email")
    if (user is None) or (email is None):
      log.error("Can't set None username in HGRC")
    else:
      hgrc.write(basedir, "ui", "username", "%s <%s>" % (user, email))

